#ifndef fe51_namespace_H
#define fe51_namespace_H

#define  fe51              crypto_scalarmult_curve25519_sandy2x_fe51
#define _fe51             _crypto_scalarmult_curve25519_sandy2x_fe51
#define  fe51_pack         crypto_scalarmult_curve25519_sandy2x_fe51_pack
#define _fe51_pack        _crypto_scalarmult_curve25519_sandy2x_fe51_pack
#define  fe51_mul          crypto_scalarmult_curve25519_sandy2x_fe51_mul
#define _fe51_mul         _crypto_scalarmult_curve25519_sandy2x_fe51_mul
#define  fe51_nsquare      crypto_scalarmult_curve25519_sandy2x_fe51_nsquare
#define _fe51_nsquare     _crypto_scalarmult_curve25519_sandy2x_fe51_nsquare

#define  fe51_invert       crypto_scalarmult_curve25519_sandy2x_fe51_invert

#endif /* ifndef fe51_namespace_H */

