/* Minimal quirks.h for Keccak/SHAKE/TurboSHAKE symbol prefixing */

#ifndef quirks_H
#ifndef NO_QUIRKS

#define keccak1600_armsha3_extract_bytes _sodium_keccak1600_armsha3_extract_bytes
#define keccak1600_armsha3_init _sodium_keccak1600_armsha3_init
#define keccak1600_armsha3_permute_12 _sodium_keccak1600_armsha3_permute_12
#define keccak1600_armsha3_permute_24 _sodium_keccak1600_armsha3_permute_24
#define keccak1600_armsha3_xor_bytes _sodium_keccak1600_armsha3_xor_bytes
#define keccak1600_ref_extract_bytes _sodium_keccak1600_ref_extract_bytes
#define keccak1600_ref_init _sodium_keccak1600_ref_init
#define keccak1600_ref_permute_12 _sodium_keccak1600_ref_permute_12
#define keccak1600_ref_permute_24 _sodium_keccak1600_ref_permute_24
#define keccak1600_ref_xor_bytes _sodium_keccak1600_ref_xor_bytes
#define mlkem768_ref_dec _sodium_mlkem768_ref_dec
#define mlkem768_ref_enc _sodium_mlkem768_ref_enc
#define mlkem768_ref_enc_deterministic _sodium_mlkem768_ref_enc_deterministic
#define mlkem768_ref_keypair _sodium_mlkem768_ref_keypair
#define mlkem768_ref_seed_keypair _sodium_mlkem768_ref_seed_keypair
#define shake128_ref _sodium_shake128_ref
#define shake128_ref_init _sodium_shake128_ref_init
#define shake128_ref_init_with_domain _sodium_shake128_ref_init_with_domain
#define shake128_ref_squeeze _sodium_shake128_ref_squeeze
#define shake128_ref_update _sodium_shake128_ref_update
#define shake256_ref _sodium_shake256_ref
#define shake256_ref_init _sodium_shake256_ref_init
#define shake256_ref_init_with_domain _sodium_shake256_ref_init_with_domain
#define shake256_ref_squeeze _sodium_shake256_ref_squeeze
#define shake256_ref_update _sodium_shake256_ref_update
#define turboshake128_ref _sodium_turboshake128_ref
#define turboshake128_ref_init _sodium_turboshake128_ref_init
#define turboshake128_ref_init_with_domain _sodium_turboshake128_ref_init_with_domain
#define turboshake128_ref_squeeze _sodium_turboshake128_ref_squeeze
#define turboshake128_ref_update _sodium_turboshake128_ref_update
#define turboshake256_ref _sodium_turboshake256_ref
#define turboshake256_ref_init _sodium_turboshake256_ref_init
#define turboshake256_ref_init_with_domain _sodium_turboshake256_ref_init_with_domain
#define turboshake256_ref_squeeze _sodium_turboshake256_ref_squeeze
#define turboshake256_ref_update _sodium_turboshake256_ref_update

#endif
#endif
