#ifndef x25519_ref10_H
#define x25519_ref10_H

#include "crypto_scalarmult_curve25519.h"
#include "../scalarmult_curve25519.h"

extern struct crypto_scalarmult_curve25519_implementation
    crypto_scalarmult_curve25519_ref10_implementation;

#endif
